import peasy.*;
import processing.data.Table;
import processing.data.TableRow;
import controlP5.*;

PeasyCam cam;
ControlP5 cp5;

//GUI things
boolean showGasGiant = true;
boolean showNeptuneLike = true;
boolean showSuperEarth=true;
boolean showTerrestrial = true;
boolean showOrbits = true;
boolean showCurves = true;

ArrayList<Exoplanet> exoplanets;

// Global variables for data range
int minYear = Integer.MAX_VALUE, maxYear = Integer.MIN_VALUE;
int currentYearFilter = 2023; //for slider only
float minMagnitude = Float.MAX_VALUE, maxMagnitude = Float.MIN_VALUE;
float minDistance = Float.MAX_VALUE, maxDistance = Float.MIN_VALUE;

float innerRadius = 50;
float outerRadius = 1000;

final float EARTH_RADIUS = 4; //made 4 times bigger to see better most of the planets
final float JUPITER_RADIUS = 11.21; // Jupiter is approximately 11.21 times the size of Earth


void setup() {
    size(1920, 1080, P3D);
    cam = new PeasyCam(this, 2500);
   
    cp5 =new ControlP5(this);    
     
    setGui();
    
    exoplanets = new ArrayList<Exoplanet>();
    
    Table table = loadTable("NASA_Exoplanets.csv", "header");
    for (TableRow row : table.rows()) {
        String name = row.getString("name");
        int discoveryYear = row.getInt("discovery_year");
        float stellarMagnitude = row.getFloat("stellar_magnitude");
        float distance = row.getFloat("distance");
        String planet_type = row.getString("planet_type");
        float orbital_radius = row.getFloat("orbital_radius");
        float orbital_period = row.getFloat("orbital_period");
        float eccentricity = row.getFloat("eccentricity");        
        String radius_wrt = row.getString("radius_wrt");
        float radius_multiplier = row.getFloat("radius_multiplier");
        
        exoplanets.add(new Exoplanet(name, discoveryYear, stellarMagnitude, distance,planet_type,orbital_radius, orbital_period,eccentricity,radius_wrt, radius_multiplier));
    }
}

void draw() {
    background(0);
    lights();
          
    if (showCurves) {    
    drawYearCircle(minYear, maxYear, 1000);
    drawMagnitudeCircle(minMagnitude, maxMagnitude, 1000); 
    drawDistanceCircle(minDistance, maxDistance, 1000);
    }
    
    for (Exoplanet planet : exoplanets) {
      if (planet.discoveryYear <= currentYearFilter) {
        if ((planet.planetType == PlanetType.GAS_GIANT && showGasGiant) ||
          (planet.planetType == PlanetType.NEPTUNE_LIKE && showNeptuneLike) ||
          (planet.planetType == PlanetType.SUPER_EARTH && showSuperEarth) ||
          (planet.planetType == PlanetType.TERRESTRIAL && showTerrestrial)) {   
          drawPlanet(planet);
          
          if (showOrbits) {
            drawOrbit(planet);
          }
        }
     }
  } 
   gui();
  
}


void drawPlanet(Exoplanet planet) {
     if (planet != null) {
        float theta = map(planet.discoveryYear, minYear, maxYear, 0, TWO_PI);
        float phi = map(planet.stellarMagnitude, minMagnitude, maxMagnitude, 0, PI);
        float logDistance = log(planet.distance);
        float r = map(logDistance, log(minDistance), log(maxDistance), innerRadius, outerRadius);

        float x = r * sin(phi) * cos(theta);
        float y = r * sin(phi) * sin(theta);
        float z = r * cos(phi);

        float baseRadius = planet.radius_wrt.equals("Jupiter") ? JUPITER_RADIUS : EARTH_RADIUS;
        float strokeWeightValue = planet.radius_multiplier * baseRadius;
        
        pushMatrix();
        translate(x, y, z);
        if (planet.planetType != null) {
            stroke(getColorForPlanetType(planet.planetType));
            strokeWeight(strokeWeightValue); // Set the stroke weight based on the planet size
        }
        // Use a default stroke weight if planetType is null or not applicable
        else {
            stroke(255); // Default color, e.g., white
            strokeWeight(2); // Default stroke weight
        }
        point(0, 0, 0); // Draw the planet as a point at the translated location  
        float screenX = screenX(0, 0, 0);
        float screenY = screenY(0, 0, 0);
        popMatrix();
        
        if (dist(mouseX, mouseY, screenX, screenY) < 10) {
        cp5.get(Textlabel.class, "dynamicName").setText("Exoplanet Name: " + planet.name.toUpperCase());
        cp5.get(Textlabel.class, "dynamicDistance").setText("Distance: " + planet.distance + " ly");
        cp5.get(Textlabel.class, "dynamicYear").setText("Discovery Year: " + planet.discoveryYear);  
     }  
  } 
}
 
void drawOrbit(Exoplanet planet) {
    pushMatrix();
        
    // Calculate the position of the planet to correctly place the orbit
    float theta = map(planet.discoveryYear, minYear, maxYear, 0, TWO_PI);
    float phi = map(planet.stellarMagnitude, minMagnitude, maxMagnitude, 0, PI);
    float r = map(log(planet.distance), log(minDistance), log(maxDistance), innerRadius, outerRadius);

    float x = r * sin(phi) * cos(theta);
    float y = r * sin(phi) * sin(theta);
    float z = r * cos(phi);
    
    float orbitalPeriod = planet.orbital_period;
    float orbitalRadius = planet.orbital_radius;
    float eccentricity = planet.eccentricity;
    float timeScale = 0.1;
    float angle = (TWO_PI / orbitalPeriod) * ((frameCount * timeScale) % orbitalPeriod);
    
    
    float orbitingX = cos(angle) * planet.orbital_radius; // orbital_radius used for orbit's size
    float orbitingY = sin(angle) * planet.orbital_radius;
    
    float semiMinorAxis = orbitalRadius * sqrt(1-sq(eccentricity));
    
    // Translate to the planet's position
    translate(x, y, z);
    
    // Assuming orbits are flat (along the XY plane), no need for further rotation
    noFill();
    stroke(255,160); 
    strokeWeight(1);
    //ellipseMode(RADIUS); 
    
    ellipse(0, 0, orbitalRadius, semiMinorAxis); // 
    
    strokeWeight(5);
    stroke(255,255,255,150);
    point(orbitingX, orbitingY, 0);
    
    popMatrix();
}
